# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.


import pyuv
import zmq
import uzmq

loop = pyuv.Loop.default_loop()

ctx = zmq.Context()
s = ctx.socket(zmq.REP)
s.bind('tcp://127.0.0.1:5555')


def rep_handler(handle, events, errors):
    # We don't know how many recv's we can do?
    msg = s.recv()
    # No guarantee that we can do the send. We need a way of putting the
    # send in the event loop.
    s.send(msg)


poll = uzmq.ZMQPoll(loop, s)
poll.start(pyuv.UV_READABLE, rep_handler)

loop.run()



# -*- coding: utf-8 -
#
# This file is part of gaffer. See the NOTICE for more information.
import time

import pyuv
import zmq
from zmq.tests import BaseZMQTestCase

from uzmq import ZMQ


def wait():
    time.sleep(.25)


class TestZMQStream(BaseZMQTestCase):

    def test_simple(self):
        req, rep = self.create_bound_pair(zmq.REQ, zmq.REP)
        wait()

        loop = pyuv.Loop.default_loop()
        s = ZMQ(loop, rep)

        r = []
        def cb(stream, msg, err):
            r.append(msg[0])

        s.start_read(cb)
        req.send(b'test')
        wait()

        def stop(handle):
            s.stop()

        t = pyuv.Timer(loop)
        t.start(stop, 0.8, 0.0)

        loop.run()
        assert r == [b'test']

    def test_echo(self):
        req, rep = self.create_bound_pair(zmq.REQ, zmq.REP)
        wait()

        loop = pyuv.Loop.default_loop()
        s = ZMQ(loop, rep)
        s1 = ZMQ(loop, req)

        r = []
        def cb(stream, msg, err):
            r.append(msg[0])
            stream.write(msg[0])

        r1 = []
        def cb1(stream, msg, err):
            r1.append(msg[0])
            s.stop()
            s1.stop()

        s.start_read(cb)
        s1.start_read(cb1)
        s1.write(b'echo')
        loop.run()

        assert r == [b'echo']
        assert r1 == [b'echo']





# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.

import pyuv
import zmq
import uzmq


loop = pyuv.Loop.default_loop()

ctx = zmq.Context()
s = ctx.socket(zmq.REP)
s.bind('tcp://127.0.0.1:5555')


stream = uzmq.ZMQ(loop, s)

def echo(handle, msg, err):
    print " ".join(msg)
    stream.send_multipart(msg)

stream.start_read(echo)

loop.run()



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.

version_info = (0, 1, 0)
__version__ = ".".join(map(str, version_info))

from uzmq.poll import ZMQPoll
from uzmq.sock import ZMQ



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.

from collections import deque
import logging

import pyuv
import zmq

from .poll import ZMQPoll
from . import util

class ZMQ(object):
    """\
        :param loop: loop object where this handle runs (accessible
            through :py:attr:`Poll.loop`).
        :param int socket: zmq socket
            to be monitored for readibility or writability.

        The ZMQ handles provides qsynchronous ZMQ sockets functionnality
        both for bound and connected sockets.
    """


    def __init__(self, loop, socket):
        self.loop = loop
        self.socket = socket

        # shortcircuit some socket methods
        self.bind = self.socket.bind
        self.bind_to_random_port = self.socket.bind_to_random_port
        self.connect = self.socket.connect
        self.setsockopt = self.socket.setsockopt
        self.getsockopt = self.socket.getsockopt
        self.setsockopt_string = self.socket.setsockopt_string
        self.getsockopt_string = self.socket.getsockopt_string
        self.setsockopt_unicode = self.socket.setsockopt_unicode
        self.getsockopt_unicode = self.socket.getsockopt_unicode

        self._poll = ZMQPoll(loop, socket)
        self._events = 0

        self._send_queue = deque()
        self._read_cb = None
        self._read_copy = True
        self._read_track = False


    def start_read(self, callback, copy=True, track=False):
        """
        :param callback: callable
            callback must take exactly one argument, which will be a
            /iist, as returned by socket.recv_multipart()
            if callback is None, recv callbacks are disabled.
        :param copy: bool
            copy is passed directly to recv, so if copy is False,
            callback will receive Message objects. If copy is True,
            then callback will receive bytes/str objects.
        :param track: bool
            Should the message be tracked for notification that ZMQ has
            finished with it? (ignored if copy=True)

        Callback signature: ``callback(zmq_handle, msg, error)``.

        Start reading for incoming messages from the remote endpoint.
        """
        if not util.is_callable(callback):
            raise TypeError("a callable is required")

        self._read_cb = callback
        self._read_copy = copy
        self._read_track = track

        if not self._events & pyuv.UV_READABLE:
            self._events |= pyuv.UV_READABLE
            self._poll.start(self._events, self._on_events)


    def stop_read(self):
        """ Stop reading data from the remote endpoint. """
        self._events = self._events & (~pyuv.UV_READABLE)
        self._poll.start(self._events, self._on_events)

    def write(self, msg, flags=0, copy=True, track=False,
            callback=None):
        """\
            :param msg: object, str, Frame The content of the message

            :param flags: int
                Any supported flag

            :param copy: bool
                Should the message be tracked for notification that ZMQ
                has finished with it? (ignored if copy=True)

            :param track: bool
                Should the message be tracked for notification that ZMQ
                has finished with it? (ignored if copy=True)


            Callback signature: ``callback(zmq_handle, msg, status)``.

                Send a message.  See zmq.socket.send for details."""
        return self.write_multipart([msg], flags=flags, copy=copy,
                track=track, callback=callback)

    def write_multipart(self, msg, flags=0, copy=True, track=False,
            callback=None):
        """ :param msg: object, str, Frame, the content of the message
            :param flags: int
                Any supported flag
            :param copy: bool
                Should the message be tracked for notification that ZMQ
                has finished with it? (ignored if copy=True)
            :param track: bool
                Should the message be tracked for notification that ZMQ
                has finished with it? (ignored if copy=True)

            Callback signature: ``callback(zmq_handle, msg, status)``.

            Send a multipart message. See zmq.socket.send_multipart for
            details."""


        kwargs = dict(flags=flags, copy=copy, track=track)
        self._send_queue.append((msg, kwargs, callback))

        if not self._events & pyuv.UV_WRITABLE:
            self._events |= pyuv.UV_WRITABLE
            self._poll.start(self._events, self._on_events)

    def stop(self):
        """ Stop the ZMQ handle """
        self._poll.stop()

    def close(self):
        """Close the ZMQ handle. After a handle has been closed no other
        operations can be performed on it."""

        self._poll.close()

    def flush(self):
        """Flush pending messages.

        This method safely handles all pending incoming and/or outgoing
        messages, bypassing the inner loop, passing them to the registered
        callbacks."""
        if self._send_queue:
            while True:
                try:
                    self._send()
                except IndexError:
                    break

    def _send(self):
        res = self._send_queue.popleft()
        msg, kwargs, cb = res
        try:
            status = self.socket.send_multipart(msg, **kwargs)
        except zmq.ZMQError as e:
            logging.error("SEND Error: %s", e)
            status = e

        if util.is_callable(cb):
            cb(self, msg, status)

    def _on_events(self, handle, events, err):
        if events & pyuv.UV_READABLE:
            self._on_read()

        if events & pyuv.UV_WRITABLE:
            self._on_write()

    def _on_read(self):
        if not self._poll.active:
            return

        try:
            msg = self.socket.recv_multipart(zmq.NOBLOCK,
                    copy=self._read_copy, track=self._read_track)
        except zmq.ZMQError as e:
            if e.errno == zmq.EAGAIN:
                # state changed since poll event
                return
            else:
                logging.error("RECV Error: %s" % zmq.strerror(e.errno))
                self._read_cb(self, None, e.errno)
        else:
            self._read_cb(self, msg, None)

    def _on_write(self):
        try:
            self._send()
        except IndexError:
            pass



# -*- coding: utf-8 -
#
# This file is part of gaffer. See the NOTICE for more information.

import pyuv
import zmq
from zmq.tests import BaseZMQTestCase

import time

from uzmq import ZMQPoll

def wait():
    time.sleep(.25)

class TestPoll(BaseZMQTestCase):

    def test_simple(self):
        """Tornado poller implementation maps events correctly"""
        req,rep = self.create_bound_pair(zmq.REQ, zmq.REP)

        loop = pyuv.Loop.default_loop()
        poll = ZMQPoll(loop, rep)

        r = []
        def cb(handle, ev, error):
            r.append(ev & pyuv.UV_READABLE)
            r.append(rep.recv())

        poll.start(pyuv.UV_READABLE, cb)
        req.send(b'req')
        t = pyuv.Timer(loop)

        def stop(h):
            poll.stop()

        t = pyuv.Timer(loop)
        t.start(stop, 0.4, 0.0)
        loop.run()

        assert r == [1, b'req']

    def test_poll_rw(self):
        """Tornado poller implementation maps events correctly"""
        req,rep = self.create_bound_pair(zmq.REQ, zmq.REP)
        wait()

        loop = pyuv.Loop.default_loop()
        poll = ZMQPoll(loop, req)
        poll1 = ZMQPoll(loop, rep)

        r = []
        def cb(handle, ev, error):
            r.append(ev & pyuv.UV_READABLE)
            r.append(rep.recv())
            handle.close()

        def cb1(handle, ev, error):
            handle.stop()
            r.append(ev & pyuv.UV_WRITABLE)
            req.send(b'req')

        poll.start(pyuv.UV_READABLE, cb)
        poll1.start(pyuv.UV_WRITABLE, cb1)

        t = pyuv.Timer(loop)

        def stop(h):
            poll.stop()
            poll1.close()

        loop.run()

        assert r == [2, 1, b'req']

    def test_echo(self):
        req, rep = self.create_bound_pair(zmq.REQ, zmq.REP)
        wait()

        loop = pyuv.Loop.default_loop()
        p = ZMQPoll(loop, rep)
        p1 = ZMQPoll(loop, req)

        r = []
        r1 = []

        def cb(handle, ev, error):
            if ev & pyuv.UV_READABLE:
                data = rep.recv()
                r.append(data)


            if ev & pyuv.UV_WRITABLE:
                rep.send(r[-1])
                if len(r) == 2:
                    handle.stop()



        def cb1(handle, ev, error):
            if ev & pyuv.UV_READABLE:
                data = req.recv()
                r1.append(data)

                if len(r1) == 2:
                    handle.stop()


            if ev & pyuv.UV_WRITABLE:
                req.send(b"echo")


        p.start(pyuv.UV_READABLE | pyuv.UV_WRITABLE, cb)
        p1.start(pyuv.UV_READABLE | pyuv.UV_WRITABLE, cb1)


        req.send(b"echo")

        def stop(h):
            p.close()
            p1.close()

        loop.run()

        assert r == [b'echo', b'echo']
        assert r1 == [b'echo', b'echo']



# -*- coding: utf-8 -
#
# This file is part of uvzmq. See the NOTICE for more information.

import os
import sys

from setuptools import setup, find_packages

from uzmq import __version__

py_version = sys.version_info[:2]

if py_version < (2, 6):
    raise RuntimeError('On Python 2, Gaffer requires Python 2.6 or better')


CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: POSIX',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.0',
    'Programming Language :: Python :: 3.1',
    'Programming Language :: Python :: 3.2',
    'Topic :: System :: Boot',
    'Topic :: System :: Monitoring',
    'Topic :: System :: Systems Administration',
    'Topic :: Software Development :: Libraries']


# read long description
with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as f:
    long_description = f.read()

DATA_FILES = [
        ('uzmq', ["LICENSE", "MANIFEST.in", "NOTICE", "README.rst",
                        "THANKS", "UNLICENSE"])
        ]


setup(name='uzmq',
      version = __version__,
      description = 'libuv interface for ZeroMQ',
      long_description = long_description,
      classifiers = CLASSIFIERS,
      license = 'BSD',
      url = 'http://github.com/benoitc/uzmq',
      author = 'Benoit Chesneau',
      author_email = 'benoitc@e-engura.org',
      packages=find_packages(),
      install_requires = [
          'pyuv>=0.8.3',
          'pyzmq'
      ],
      data_files = DATA_FILES)



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.

"""
ZMQPoll: ZMQ Poll handle

"""
import pyuv
import zmq

from . import util

class ZMQPoll(object):
    """\
        :param loop: loop object where this handle runs (accessible
            through :py:attr:`Poll.loop`).
        :param int socket: zmq socket
            to be monitored for readibility or writability.

        ``ZMQPoll`` ZMQPoll handles can be used to monitor an arbitrary file
        descritor for readability or writability.  On Unix any file
        descriptor is supported but on Windows only sockets are
        supported.

        .. py:attribute:: loop

            *Read only*

            :py:class:`pyuv.Loop` object where this handle runs.

    """

    def __init__(self, loop, socket):
        self.loop = loop
        self.socket = socket

        # initialize private variable
        self.fd = socket.getsockopt(zmq.FD)
        self._poller = pyuv.Poll(loop, self.fd)

        self._callback = None
        self._started = False

    @property
    def active(self):
        """*Read only*

            Indicates if this handle is active."""
        return self._poller.active

    @property
    def closed(self):
        """*Read only*

            Indicates if this handle is closing or already closed."""
        return self._poller.closed

    def start(self, events, callback):
        """\
            :param events: int
                Mask of events that will be detected. The possible
                events are `pyuv.UV_READABLE` or `pyuv.UV_WRITABLE`.

            :param callback: callable
                Function that will be called when the ``Poll`` handle
                receives events.

            Callback signature: ``callback(poll_handle, events, errorno)``.

            Start or update the event mask of the ``ZMQPoll`` handle.
        """
        if not util.is_callable(callback):
            raise TypeError("a callable is required")

        self._callback = callback
        self._poller.start(events, self._poll)

    def stop(self):
        """ Stop the ``Poll`` handle. """
        self._poller.stop()

    def close(self, callback=None):
        """
        :param callable callback: Function that will be called after the
            ``ZMQPoll`` handle is closed.

        Close the ``ZMQPoll`` handle. After a handle has been closed no other
        operations can be performed on it.
        """
        self._poller.close()
        if util.is_callable(callback):
            callback(self)

    def _poll(self, handle, evs, errno):
        # trick to use the last state. Fix a race condition
        z_events = self.socket.getsockopt(zmq.EVENTS)

        events = 0
        if z_events & zmq.POLLIN:
            events |= pyuv.UV_READABLE

        if z_events & zmq.POLLOUT:
            events |= pyuv.UV_WRITABLE

        if not events:
            self._callback(self, evs, errno)
        else:
            self._callback(self, events, errno)
